#!/usr/bin/env python3
# Copyright 2023 - Barcelona Supercomputing Center
# Authors: Amirpasha Mozaffari
# MIT License

import yaml
from yaml.loader import SafeLoader
import os 
import argparse
import sys 
#from YAMLlUtils import read_yaml_conf_type1
#import f90nml

# It will readin the YAML file that is generated by the RunExample and generate a namcouple list before 


def append_multiple_lines_to_file(file_path, lines_to_append):
    with open(file_path, 'a') as file:
       # for line in lines_to_append:
        file.write(lines_to_append + '\n')


def add_field_to_namcouple(send_id, receive_id, timestepsec,send_grid_name, receive_grid):
    lines_to_append =   str(send_id) + " " + str(receive_id) + " 1 " + str(timestepsec) + " 1 rstas.nc EXPOUT \n" + \
                        str(send_grid_name) + " " + str(receive_grid) + " LAG=0 \n" + \
                        "P  0  P  0\n" + \
                        "SCRIPR \n" + \
                        "GAUSWGT LR SCALAR LATITUDE 1 9 2.0"

    return lines_to_append    



def create_empty_file(filename):
    try:
        with open(filename, 'w') as file:
            pass  # The file is created but remains empty
        print(f"Empty file '{filename}' created successfully.")
    except Exception as e:
        print(f"An error occurred: {e}")


def add_begin_to_namecouple(nfields,runtime):
    lines_to_append = """# =================================================================================================
# General OASIS configuration
# =================================================================================================
 $NFIELDS
    {nfields}
 $END
# -------------------------------------------------------------------------------------------------
 $RUNTIME
    {runtime}
 $END
# -------------------------------------------------------------------------------------------------
 $NLOGPRT
 0 0
 $END
# -------------------------------------------------------------------------------------------------
 $STRINGS 
# =================================================================================================
# Fields send from AMIP to IFS
# =================================================================================================
# --- AMIP forcing data --- [ifs amip]"""
    
    return(lines_to_append.format(nfields=nfields,runtime=runtime))


def add_end_to_namecouple():
    lines_to_append = """# -------------------------------------------------------------------------------------------------
$END
# ================================================================================================="""

    return lines_to_append




# Read the DataCoupler YAML

def read_DataCoupler_yaml_input(yaml_file):
    '''
    Reads the experiment yaml file 
    '''

    with open(yaml_file) as file:
        read_in = yaml.load(file, Loader=SafeLoader)
        file.close()
    
    # Model Information
    ModelNameSend       = read_in.get('ModelNameSend')
    ModelNameReceive      = read_in.get('ModelNameReceive')
    LogFileName     = read_in.get('LogFileName')
    PartitionType   = read_in.get('PartitionType')

    ## Run Information 

    RunLengthSec    = read_in.get('RunLengthSec')
    TimeStepSec     = read_in.get('TimeStepSec')
    StartYear       = read_in.get('StartYear')
    StartMonth      = read_in.get('StartMonth')
    StartDay        = read_in.get('StartDay')
    FixYear         = read_in.get('FixYear')
    GridInfo        = read_in.get('GridInfo')
    LDebug          = read_in.get('LDebug')

    # Create the log_file name and sanity-check 

    # Check the type of input in yaml file 
    if "FileInputVars" in read_in : 
        FileInputVars_readin = {}
        FileInputVars_readin   = read_in.get('FileInputVars')
        # Name of the Variable for this experiment 
        FileInputVarNames  = list(FileInputVars_readin.keys())
        # Variable configuration for this experiment 
        FileInputVarFields_conf = list(FileInputVars_readin.values())

    if "OasisOutputVars" in read_in :
        OasisOutputVars_readin = {} 
        OasisOutputVars_readin = read_in.get('OasisOutputVars')
        print("NOTE: OasisOutputVars_readin : {}".format(OasisOutputVars_readin), file=sys.stderr)
        # Name of the Variable for this experiment 
        OasisOutputVarsNames  = list(OasisOutputVars_readin.keys())

        # Variable configuration for this experiment 
        OasisOutputVars_conf = list(OasisOutputVars_readin.values())

    return ModelNameSend, ModelNameReceive, LogFileName, PartitionType, RunLengthSec, TimeStepSec, StartYear, StartMonth,\
             StartDay, FixYear, GridInfo, LDebug, FileInputVarFields_conf, OasisOutputVars_conf, OasisOutputVars_readin


# Create yaml_conf_name_ifs_toy_model 


def create_yaml_conf_name_ifs_toy_model(filename,ModelName,PartitionTypeReceive,run_length_sec,coupling_interval,nx,OasisOutputVars_readin):

    coupling_out_vars=[]
    
    coupling_in_vars = [] 
    for v in OasisOutputVars_readin.keys() :
        coupling_in_vars.append(OasisOutputVars_readin[v]['receive_id'])
    
    create_empty_file(filename)
    lines_to_append =""" # Copyright 2023 - Barcelona Supercomputing Center
# Author: Amirpasha Mozaffari
# TBD
---

# yaml metadata
model:
  ModelName     : {ModelName}
  #PartitionTypeSend : SerialPartition
  PartitionTypeReceive : {PartitionTypeReceive}

simulation:
  coupling_interval : {coupling_interval}
  run_length_sec : {run_length_sec}
  # Definition of the local partition grid cell 
  # L080_NX = 35718  # number of LAND grid cells at T159 resolution
  # L128_NX = 88838  # number of LAND grid cells at T255 resolution
  # L256_NX = 348528  # number of LAND grid cells at T511 resolution
  nx : {nx}

coupling: 
  out_vars : {out_vars}
  in_vars : {in_vars}
  restart_file : /path/ 
    """
    append_multiple_lines_to_file(filename, lines_to_append.format(ModelName=ModelName,PartitionType=PartitionType,coupling_interval=coupling_interval,run_length_sec=run_length_sec,nx=nx,out_vars=coupling_out_vars,in_vars=coupling_in_vars ))
    
    
   


if __name__ == '__main__':
    
    parser=argparse.ArgumentParser()
    parser.add_argument("--conf",type=str)
    parser.add_argument("--output",type=str)
    args = parser.parse_args()
    yaml_conf_file = args.conf
    output_file =  args.output 

    # Read the yaml file 

    ModelNameSend, ModelNameReceive, LogFileName, PartitionType, RunLengthSec, TimeStepSec, StartYear, StartMonth,   \
    StartDay, FixYear, GridInfo, LDebug, fileIn_vars_config, oasisOut_vars_config, OasisOutputVars_readin = read_DataCoupler_yaml_input(yaml_conf_file)

    number_of_fields = len(OasisOutputVars_readin)

    # Creating the namcouple file amd insert the read-in data from YAML file 
    create_empty_file(output_file)
    lines=add_begin_to_namecouple(number_of_fields,RunLengthSec)
    append_multiple_lines_to_file(output_file, lines)
    
    for v in OasisOutputVars_readin.keys() :
        lines = add_field_to_namcouple(OasisOutputVars_readin[v]['send_id'], OasisOutputVars_readin[v]['receive_id'], \
                                    TimeStepSec,OasisOutputVars_readin[v]['send_grid_name'], OasisOutputVars_readin[v]['receive_grid_name'])
        append_multiple_lines_to_file(output_file, lines)

    lines = add_end_to_namecouple()
    append_multiple_lines_to_file(output_file, lines)

    # Creating the yaml_conf_name_ifs_toy_model 
    
    ifs_conf_yaml = "yaml_conf_name_ifs_toy_model.yaml"
    create_yaml_conf_name_ifs_toy_model(ifs_conf_yaml,ModelNameSend,PartitionType, RunLengthSec, TimeStepSec,GridInfo,OasisOutputVars_readin)
